# %% bcmfR R-source %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# _________________________________________________________________________
#
#    bcmf_bmfmod_33.R
# _________________________________________________________________________

# === [] ====================
# ___ () ____________________

# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@


 
# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@


# /////////////////



# === [BootModel_Dietary] ====================
# ~~~ Rubin's Bayesian Bootstrap, homemade
BootModel_Dietary <- function(datacfish = DATA.Dietary.Work,
                              paramfit  = BMF.ParamFit,
                              numboot   = NUM.Boot,
                              numcontrol= NUM.Control.nls,
                              randseed  = 3001,
                              savefit   = TRUE) {
  
  startfit <- paramfit
  names(startfit) <- c("fitc0d", "fitk2")
  
  #datacfish <- get("DATA.Dietary.Work", envir = .GlobalEnv)
  
  # -- declare one extra for the base run
  fitc0darr <- numeric(numboot)
  fitk2arr  <- numeric(numboot)
  
  # -- store the base run in the first line of the array
  fitc0darr[1] <- paramfit["c0d"]
  fitk2arr[1]  <- paramfit["k2"]
 
  nrows <- length(row.names(datacfish))
  
  # declare weight array as numboot x nrows
  weightsarr <- matrix(nrow = numboot, ncol = nrows)
  weightsarr[1, ] <- 1/nrows
  
  # number of boot runs not converged set to zero
  num.notconverged <- 0
  
  # set random seed for Bayesian bootstrap
  set.seed(randseed)
  
  
  for (i in 2:numboot) {
    
    repeat {
    
      # -- draw Bayesian weights (Bayesian Bootstrap)
      bb <- runif(nrows - 1)
      bbplus <- c(0., bb, 1.)
      bbsort <- sort(bbplus)
      newweights <- diff(bbsort)
      
    
      fitbest.out <- try(nls(cfish.data ~ RunModel_Dietary(time.data,
                                                           fitc0d, fitk2),
                         data    = datacfish,
                         start   = startfit,
                         weights = newweights,
                         control = numcontrol,
                         trace   = FALSE), silent = TRUE)
      
      if (class(fitbest.out)!="try-error")  {
        
        # converged, store results (store output list as well?)
        
        parfiti    <- coef(fitbest.out)
        weightsarr[i, ] <- newweights
        
        fitc0darr[i] <- parfiti[1]
        fitk2arr[i] <- parfiti[2]
        
        break
        
      }  else  {
        
        num.notconverged <- num.notconverged + 1
        
        if (num.notconverged == numboot) stop()
        
        print(num.notconverged)
        
      }
    }
  }
    

  # -- put into dataframe
  nlsboot <- data.frame(c0d.boot = fitc0darr, k2.boot = fitk2arr)
  
  if (savefit) {
    assign("BMF.ParamBoot", nlsboot, envir = .GlobalEnv)
  }
  
  top    <- head(nlsboot, n = 10)
  bottom <- tail(nlsboot, n = 10)
  
  print(top)
  cat("   ...")
  cat("\n")
  print(bottom)
  
  # return(nlsboot)

}
# ___ (BootModel_Dietary) ____________________



# === [BootModel_Dietary_BoxCox] ====================
# ~~~ Rubin's Bayesian Bootstrap, homemade
# ~~~ assumes bc.cfish.data to be precalculated in DATA.Dietary.Work
BootModel_Dietary_BoxCox <- function(datacfish = DATA.Dietary.Work,
                                     paramfit  = BMF.ParamFit.BoxCox,
                                     lambda    = BMF.ParamFit.Lambda,
                                     lnstarter = 1e-9,
                                     numboot   = NUM.Boot,
                                     numcontrol= NUM.Control.nls,
                                     randseed  = 3001,
                                     savefit   = TRUE) {
  
  
  startfit <- paramfit
  names(startfit) <- c("fitc0d", "fitk2")
  
  # datacfish <- get("DATA.Dietary.Work", envir = .GlobalEnv)

  fitc0darr <- numeric(numboot)
  fitk2arr  <- numeric(numboot)
  
  # -- store the base run in the first line of the array
  fitc0darr[1] <- paramfit["c0d"]
  fitk2arr[1] <- paramfit["k2"]
  
  nrows <- length(row.names(datacfish))
  
  # declare weight array as numboot x nrows
  weightsarr <- matrix(nrow = numboot, ncol = nrows)
  weightsarr[1, ] <- 1/nrows
  
  # number of boot runs not converged set to zero
  num.notconverged <- 0

  # set random seed for Bayesian bootstrap
  set.seed(randseed)
  
  
  for (i in 2:numboot) {
    
    repeat {
    
      # -- draw Bayesian weights (Bayesian Bootstrap)
      bb <- runif(nrows - 1)
      bbplus <- c(0., bb, 1.)
      bbsort <- sort(bbplus)
      newweights <- diff(bbsort)  
      
  
      fitbestbc.out <- try(nls(bc.cfish.data ~ car::bcPower(RunModel_Dietary(time.data,
                                                            fitc0d, fitk2) + lnstarter,
                                                            lambda),
                   data    = datacfish,
                   start   = startfit,
                   weights = newweights,
                   control = numcontrol,
                   trace   = FALSE), silent = TRUE)
      
      if (class(fitbestbc.out)!="try-error")  {
        
        # converged, store results (store output list as well?)
        
        parfiti    <- coef(fitbestbc.out)
        weightsarr[i, ] <- newweights
        
        fitc0darr[i] <- parfiti[1]
        fitk2arr[i]  <- parfiti[2]
        
        break
        
      }  else  {
        
        num.notconverged <- num.notconverged + 1
        
        if (num.notconverged == numboot) stop()
        
        print(num.notconverged)
        
      }
    }
  }
      
      
  # -- put into dataframe
  nlsboot <- data.frame(c0d.boot.bc = fitc0darr, k2.boot.bc = fitk2arr)
  
  if (savefit) {
    assign("BMF.ParamBoot.BoxCox", nlsboot, envir = .GlobalEnv)
  }
  
  top    <- head(nlsboot, n = 10)
  bottom <- tail(nlsboot, n = 10)
  
  print(top)
  cat("   ...")
  cat("\n")
  print(bottom)
  
  # return(nlsboot)
  
}
# ___ (BootModel_Dietary_BoxCox) ____________________


# /////////////////
# === [BootModel_Dietary_Ln] ====================
# ~~~ Rubin's Bayesian Bootstrap, homemade
# ~~~ assumes ln.cfish.data to be precalculated in DATA.Dietary.Work
BootModel_Dietary_Ln <- function(datacfish = DATA.Dietary.Work,
                                 paramfit  = BMF.ParamFit.Ln,
                                 lnstarter = 1e-9,
                                 numboot   = NUM.Boot,
                                 numcontrol= NUM.Control.nls,
                                 randseed  = 3001,
                                 savefit   = TRUE) {
  
  startfit <- paramfit
  
  names(startfit) <- c("fitc0d", "fitk2")
  
  # datacfish <- get("DATA.Dietary.Work", envir = .GlobalEnv)
  
  fitc0darr <- numeric(numboot)
  fitk2arr  <- numeric(numboot)
  
  # -- store the base run in the first line of the array
  fitc0darr[1] <- paramfit["c0d"]
  fitk2arr[1] <- paramfit["k2"]
  
  nrows <- length(row.names(datacfish))
  
  # declare weight array as numboot x nrows
  weightsarr <- matrix(nrow = numboot, ncol = nrows)
  weightsarr[1, ] <- 1/nrows
  
  # number of boot runs not converged set to zero
  num.notconverged <- 0
  
  # set random seed for Bayesian bootstrap
  set.seed(randseed)

  for (i in 2:numboot) {
    
    repeat {
      
      # -- draw Bayesian weights (Bayesian Bootstrap)
      bb <- runif(nrows - 1)
      bbplus <- c(0., bb, 1.)
      bbsort <- sort(bbplus)
      newweights <- diff(bbsort)
      
    
      fitbestln.out <- try(nls(ln.cfish.data ~ log(RunModel_Dietary(time.data,
                                                        fitc0d, fitk2) + lnstarter),
                   data    = datacfish,
                   start   = startfit,
                   weights = newweights,
                   control = numcontrol,
                   trace   = FALSE), silent = TRUE)
    
      if (class(fitbestln.out)!="try-error")  {
        
        # converged, store results (store output list as well?)
        
        parfiti    <- coef(fitbestln.out)
        weightsarr[i, ] <- newweights
        
        fitc0darr[i] <- parfiti[1]
        fitk2arr[i]  <- parfiti[2]
        
        break
        
      }  else  {
        
        num.notconverged <- num.notconverged + 1
        
        if (num.notconverged == numboot) stop()
        
        print(num.notconverged)
        
      }
    }
  }
      
  
  # -- put into dataframe   
  nlsboot <- data.frame(c0d.boot.ln = fitc0darr, k2.boot.ln = fitk2arr)
  

  if (savefit) {
    assign("BMF.ParamBoot.Ln", nlsboot, envir = .GlobalEnv)
  }
  
  top    <- head(nlsboot, n = 10)
  bottom <- tail(nlsboot, n = 10)
  
  print(top)
  cat("   ...")
  cat("\n")
  print(bottom)
  
  # return(nlsboot)
  
}
# ___ (BootModel_Dietary_Ln) ____________________



# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

# === [FitModel_Dietary] ====================
# --- FINAL bcmfR Untransformed nonlinear fit
FitModel_Dietary <- function(datacfish = DATA.Dietary.Work,
                             testparam = TEST.Dietary.Design,
                             numcontrol= NUM.Control.nls,
                             lnstarter = 1e-9,
                             startfit  = NULL, # use vector
                             trace     = FALSE,
                             savefit   = TRUE) {
  
  if (is.null(startfit)) {
    # returns fitc0d, fitk2 vector
    startfit <- StartFit_Dietary(datacfish = datacfish,
                                 #testparam = testparam,
                                 lnstarter = lnstarter)
  } else {
    # if names lack, add them
    if (is.null(names(startfit)))
      names(startfit) <- c("fitc0d", "fitk2")
  }
  
  
  if (savefit)
    assign("BMF.StartFit", startfit, envir = .GlobalEnv)
  
  modfitcfish = nls(cfish.data ~ 
                      RunModel_Dietary(time.data, fitc0d, fitk2),
                        data    = datacfish,
                        start   = startfit,
                        control = numcontrol,
                        trace   = trace)
  
  if (savefit) {
    assign("BMF.ModelFit", modfitcfish, envir = .GlobalEnv)
  }
  
  coefvec <- coef(modfitcfish)
  names(coefvec) <- c("c0d", "k2")
  
  if (savefit) {
    # only ModelFit does this
    assign("BMF.ParamFit", coefvec, envir = .GlobalEnv)
  }
  
  return(summary(modfitcfish))
  
}
# ___ (FitModel_Dietary) ____________________


# /////////////////

# === [FitModel_Dietary_BoxCox] ====================
# --- FINAL bcmfR
FitModel_Dietary_BoxCox <- function(datacfish = DATA.Dietary.Work,
                                    testparam = TEST.Dietary.Design,
                                    lambda    = BMF.LambdaEst["fit"],
                                    numcontrol= NUM.Control.nls,
                                    lnstarter = 1e-9,
                                    startfit  = NULL, # use vector
                                    trace     = FALSE,
                                    savefit   = TRUE) {
  #..........
  
  if (is.null(startfit)) {
    # returns fitc0d, fitk2 vector
    startfit <- StartFit_Dietary(datacfish = datacfish,
                                 #testparam = testparam,
                                 lnstarter = lnstarter)
  } else {
    if (is.null(names(startfit)))
      names(startfit) <- c("fitc0d", "fitk2")
  }
  
  #-- save the startfit vector, whether input, or calculated
  if (savefit)
    assign("BMF.StartFit", startfit, envir = .GlobalEnv)
  
  # ..........
  
  # -- precalculate boxcox-transform in DATA.Dietary.Work
  datacfish <- dataTrans_BoxCox(datacfish,
                                lambda = lambda,
                                lnstarter = lnstarter)
  
  # -- save transform in work sheet (bc.cfish.data)
  if (savefit)
    assign("DATA.Dietary.Work", datacfish, envir = .GlobalEnv)  
  
  # ..........
  
  # -- 'both sides' transform: reintroduced lnstarter in RHS
  modfitcfishbc = nls(bc.cfish.data ~ 
                        car::bcPower(RunModel_Dietary(time.data,
                                                 fitc0d, fitk2) + lnstarter,
                                lambda),
                      data    = datacfish,
                      start   = startfit,
                      control = numcontrol,
                      trace   = trace)
  
  if (savefit) {
    assign("BMF.ModelFit.BoxCox", modfitcfishbc, envir = .GlobalEnv)
  }
  
  # ..........
  
  # -- rename the coef vector for paramFit
  coefvecbc <- coef(modfitcfishbc)
  names(coefvecbc) <- c("c0d", "k2")
  
  if (savefit) {
    # only modelFit does this
    assign("BMF.ParamFit.BoxCox", coefvecbc, envir = .GlobalEnv)
    assign("BMF.ParamFit.Lambda", lambda, envir = .GlobalEnv)
  }
  
  return(summary(modfitcfishbc))
  
}
# ___ (FitModel_Dietary_BoxCox) ____________________


# /////////////////


# === [FitModel_Dietary_Ln] ====================
# --- FINAL bcmfR
FitModel_Dietary_Ln <- function(datacfish = DATA.Dietary.Work,
                                testparam = TEST.Dietary.Design,
                                numcontrol= NUM.Control.nls,
                                lnstarter = 1e-9,
                                startfit  = NULL, # use vector
                                trace     = FALSE,
                                savefit   = TRUE) {
  
  if (is.null(startfit)) {
    # returns fitc0d, fitk2 vector
    startfit <- StartFit_Dietary(datacfish = datacfish,
                                 #testparam = testparam,
                                 lnstarter = lnstarter)
  } else {
    if (is.null(names(startfit)))
      names(startfit) <- c("fitc0d", "fitk2")
  }
  
  #-- save the startfit vector, whether input, or calculated
  if (savefit)
    assign("BMF.StartFit", startfit, envir = .GlobalEnv)
  
  #..........
  
  #-- precalculate ln-transform in DATA.Dietary.Work
  datacfish <- dataTrans_Ln(datacfish, lnstarter = lnstarter)
  
  #-- save transform in work sheet (ln.cfish.data)
  if (savefit)
    assign("DATA.Dietary.Work", datacfish, envir = .GlobalEnv)
  
  # ..........
  

  #-- 'both sides' transform: reintroduced lnstarter in RHS
  modfitcfishln = nls(ln.cfish.data ~ 
                        log(RunModel_Dietary(time.data,
                                             fitc0d, fitk2) + lnstarter),
                      data  = datacfish,
                      start = startfit,
                      control = numcontrol,
                      trace = trace)
  # ..........
  
  # -- keep fitc0d and fitk2 names
  if (savefit) {
    assign("BMF.ModelFit.Ln", modfitcfishln, envir = .GlobalEnv)
  }
  
  #-- rename the ParamFit
  coefvecln <- coef(modfitcfishln)
  names(coefvecln) <- c("c0d", "k2")
  
  if (savefit) {
    # only modelFit does this
    assign("BMF.ParamFit.Ln", coefvecln, envir = .GlobalEnv)
  }
  
  return(summary(modfitcfishln))
  
}
# ___ (FitModel_Dietary_Ln) ____________________



# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

# === [ModelDiagnostics_Dietary] ====================
# --- package nlstools
ModelDiagnostics_Dietary <- function(modelfit = BMF.ModelFit) {
  nlsresun <- nlstools::nlsResiduals(modelfit)
  plot(nlsresun)
  nlstools::test.nlsResiduals(nlsresun)  
}
# ___ (ModelDiagnostics_Dietary) ____________________


# /////////////////

# === [ModelDiagnostics_Dietary_BoxCox] ====================
# --- package nlstools
ModelDiagnostics_Dietary_BoxCox <- function(modelfit = BMF.ModelFit.BoxCox) {
  nlsresbc <- nlstools::nlsResiduals(modelfit)
  plot(nlsresbc)
  nlstools::test.nlsResiduals(nlsresbc)  
}
# ___ (ModelDiagnostics_Dietary_BoxCox) ____________________


# /////////////////

# === [ModelDiagnostics_Dietary_Ln] ====================
# --- package nlstools
ModelDiagnostics_Dietary_Ln <- function(modelfit = BMF.ModelFit.Ln) {
  nlsresln <- nlstools::nlsResiduals(modelfit)
  plot(nlsresln)
  nlstools::test.nlsResiduals(nlsresln)  
}
# ___ (ModelDiagnostics_Dietary_Ln) ____________________






# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@













# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

# === [ModelTrans_BoxCox_Dietary] ====================
ModelTrans_BoxCox_Dietary <- function(modelfit = BMF.ModelFit) {
  
  if (exists("BMF.ModelFit"))
    modelfit <- BMF.ModelFit
  else
    modelfit <- FitModel_Dietary()
  
  lambdaest <- ModelTrans_BoxCox(modelfit)   # zit in base.R
  
  assign("BMF.LambdaEst", lambdaest, envir = .GlobalEnv)
  
  return(lambdaest)
  
}
# ___ (ModelTrans_BoxCox_Dietary) ____________________



# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

# //////////////////////////////////////////////
# === [PlotBestFit_Dietary] ====================
# ~~~ assumes BMF.BestFit to exist, saved by RunBestFit_Dietary()
PlotBestFit_Dietary <- function(testdata    = DATA.Dietary.Work,
                                testmeas    = TEST.Dietary.Measured,
                                bestfitdata = BMF.BestFit,
                                plotstyle   = PLOT.StyleFit,
                                plottheme   = PLOT.Theme,
                                plottitle   = PLOT.Title.BestFit,
                                plotxlab    = NULL,
                                plotylab    = NULL) {
  
  if (is.null(plotxlab)) {
    # -- x label text
    plotxlab <- paste(testmeas["time","name"], "  (",
                      testmeas["time","unit"], ")", sep = "")
  }
  
  if (is.null(plotylab)) {
    # -- y label text
    plotylab <- paste(testmeas["cfish","name"], "  (",
                      testmeas["cfish","unit"], ")", sep = "")
  }
  
  #-- scatter of measured test data
  plot0 <- ggplot2::ggplot(testdata, ggplot2::aes(time.data, cfish.data))
  
  # -- add scatter and bestfit curve

  plot1 <- plot0 +
      
      ggplot2::geom_line(data = bestfitdata,
                ggplot2::aes(time.best, cfish.best),
                
                size = plotstyle$linebest$size,
                linetype = plotstyle$linebest$style,
                color = plotstyle$linebest$color) +
    
      ggplot2::geom_point(shape = plotstyle$symbol$shape,
                 size  = plotstyle$symbol$size,
                 color = plotstyle$symbol$color) +
    
      ggplot2::ggtitle(plottitle) +
    
      ggplot2::labs(x = plotxlab, y = plotylab)
  
  # -- add theme 
  plot2 <- plot1 + expand_theme(plottheme)
  
  print(plot2)
  
}
# ___ (PlotBestFit_Dietary) ____________________


# /////////////////////////////////////////////////////
# === [PlotBestFit_Dietary_BoxCox] ====================
# ~~~ assumes BMF.BestFit.Ln to exist, saved by RunBestFit_Dietary()
# ~~~ assumes bc.cfish.data in DATA.Dietary.Work
PlotBestFit_Dietary_BoxCox <- function(testdata    = DATA.Dietary.Work,
                                       testmeas    = TEST.Dietary.Measured,
                                       bestfitdata = BMF.BestFit.BoxCox,
                                       plotstyle   = PLOT.StyleFit.BoxCox,
                                       plottheme   = PLOT.Theme,
                                       plottitle   = PLOT.Title.BestFit.BoxCox,
                                       plotxlab    = NULL,
                                       plotylab    = NULL) {

  if (is.null(plotxlab)) {
    # -- x label text
    plotxlab <- paste(testmeas["time","name"], "  (",
                      testmeas["time","unit"], ")", sep = "")
  }
  
  if (is.null(plotylab)) {
    # -- y label text
    plotylab <- paste("Box-Cox ", testmeas["cfish","name"], "  (",
                      testmeas["cfish","unit"], ")", sep = "")
  }
  
  #-- scatter of measured test data
  plot0 <- ggplot2::ggplot(testdata, ggplot2::aes(time.data, bc.cfish.data))
  
  # -- add scatter and bestfit curve
  plot1 <- plot0 +
           
      ggplot2::geom_line(data = bestfitdata,
                ggplot2::aes(time.best, bc.cfish.best),
                
                size = plotstyle$linebest$size,
                linetype = plotstyle$linebest$style,
                color = plotstyle$linebest$color) +

      ggplot2::geom_point(shape = plotstyle$symbol$shape,
                 size  = plotstyle$symbol$size,
                 color = plotstyle$symbol$color) +
    
      ggplot2::ggtitle(plottitle) +
    
      ggplot2::labs(x = plotxlab, y = plotylab)
  
  # -- add theme 
  plot2 <- plot1 + expand_theme(plottheme)
  
  print(plot2)
  
}
# ___ (PlotBestFit_Dietary_BoxCox) ____________________


# /////////////////
# === [PlotBestFit_Dietary_Ln] ====================
# ~~~ assumes BMF.BestFit.Ln to exist, saved by RunBestFit_Dietary()
# ~~~ assumes ln.cfish.data in DATA.Dietary.Work
PlotBestFit_Dietary_Ln <- function(testdata    = DATA.Dietary.Work,
                                   testmeas    = TEST.Dietary.Measured,
                                   bestfitdata = BMF.BestFit.Ln,
                                   plotstyle   = PLOT.StyleFit.Ln,
                                   plottheme   = PLOT.Theme,
                                   plottitle   = PLOT.Title.BestFit.Ln,
                                   plotxlab    = NULL,
                                   plotylab    = NULL) {

  if (is.null(plotxlab)) {
    # -- x label text
    plotxlab <- paste(testmeas["time","name"], "  (",
                      testmeas["time","unit"], ")", sep = "")
  }
  
  if (is.null(plotylab)) {
    # -- y label text
    plotylab <- paste("Ln ", testmeas["cfish","name"], "  (",
                      testmeas["cfish","unit"], ")", sep = "")
  }
  
  #-- scatter of measured test data
  plot0 <- ggplot2::ggplot(testdata, ggplot2::aes(time.data, ln.cfish.data))
  
  # -- add scatter and bestfit curve
  plot1 <- plot0 +
    
      ggplot2::geom_line(data = bestfitdata,
                ggplot2::aes(time.best, ln.cfish.best),
                
                size = plotstyle$linebest$size,
                linetype = plotstyle$linebest$style,
                color = plotstyle$linebest$color) +
    
      ggplot2::geom_point(shape = plotstyle$symbol$shape,
                 size  = plotstyle$symbol$size,
                 color = plotstyle$symbol$color) +
    
      ggplot2::ggtitle(plottitle) +
    
      ggplot2::labs(x = plotxlab, y = plotylab)
  
  # -- add theme 
  plot2 <- plot1 + expand_theme(plottheme)
  
  print(plot2)
  
}
# ___ (PlotBestFit_Dietary_Ln) ____________________



# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@


# //////////////////////////////////////////////
# === [PlotConfFit_Dietary] ====================
# ~~~ assumes BMF.BestFit to exist, saved by RunBestFit_Dietary()
PlotConfFit_Dietary <- function(testdata    = DATA.Dietary.Work,
                                testmeas    = TEST.Dietary.Measured,
                                bestfitdata = BMF.BestFit,
                                conffitdata = BMF.ConfFit,
                                plotstyle   = PLOT.StyleFit,
                                plottheme   = PLOT.Theme,
                                plottitle   = PLOT.Title.ConfFit,
                                plotxlab    = NULL,
                                plotylab    = NULL) {
  
  if (is.null(plotxlab)) {
    # -- x label text
    plotxlab <- paste(testmeas["time","name"], "  (",
                      testmeas["time","unit"], ")", sep = "")
  }
  
  if (is.null(plotylab)) {
    # -- y label text
    plotylab <- paste(testmeas["cfish","name"], "  (",
                      testmeas["cfish","unit"], ")", sep = "")
  }
  
  #-- scatter of measured test data
  plot0 <- ggplot2::ggplot(testdata, ggplot2::aes(time.data, cfish.data))
  
  # -- add scatter and bestfit curve
  plot1 <- plot0 +
    
      ggplot2::geom_line(data = bestfitdata,
                ggplot2::aes(time.best, cfish.best),
                
                size = plotstyle$linebest$size,
                linetype = plotstyle$linebest$style,
                color = plotstyle$linebest$color) +
    
      ggplot2::geom_line(data = conffitdata,
                ggplot2::aes(time.conf, cfish.conflo),
                    
                size = plotstyle$lineconf$size,
                linetype = plotstyle$lineconf$style,
                color = plotstyle$lineconf$color) +
    
      ggplot2::geom_line(data = conffitdata,
                ggplot2::aes(time.conf, cfish.confup),
                
                size = plotstyle$lineconf$size,
                linetype = plotstyle$lineconf$style,
                color = plotstyle$lineconf$color) +
    
      ggplot2::geom_point(shape = plotstyle$symbol$shape,
                 size  = plotstyle$symbol$size,
                 color = plotstyle$symbol$color) +
    
      ggplot2::ggtitle(plottitle) +
    
      ggplot2::labs(x = plotxlab, y = plotylab)
  
  # -- add theme 
  plot2 <- plot1 + expand_theme(plottheme)
  
  print(plot2)
  
}
# ___ (PlotConfFit_Dietary) ____________________


# //////////////////////////////////////////////
# === [PlotConfFit_Dietary_BoxCox] ====================
# ~~~ assumes BMF.BestFit to exist, saved by RunBestFit_Dietary()
PlotConfFit_Dietary_BoxCox <- function(testdata    = DATA.Dietary.Work,
                                       testmeas    = TEST.Dietary.Measured,
                                       bestfitdata = BMF.BestFit.BoxCox,
                                       conffitdata = BMF.ConfFit.BoxCox,
                                       plotstyle   = PLOT.StyleFit.BoxCox,
                                       plottheme   = PLOT.Theme,
                                       plottitle   = PLOT.Title.ConfFit.BoxCox,
                                       plotxlab    = NULL,
                                       plotylab    = NULL) {
  
  if (is.null(plotxlab)) {
    # -- x label text
    plotxlab <- paste(testmeas["time","name"], "  (",
                      testmeas["time","unit"], ")", sep = "")
  }
  
  if (is.null(plotylab)) {
    # -- y label text
    plotylab <- paste("BoxCox ", testmeas["cfish","name"], "  (",
                      testmeas["cfish","unit"], ")", sep = "")
  }
  
  #-- scatter of measured test data
  plot0 <- ggplot2::ggplot(testdata, ggplot2::aes(time.data, bc.cfish.data))
  
  # -- add scatter, bestfit curve, and conflo & confup curves
  plot1 <- plot0 +
    
      ggplot2::geom_line(data = bestfitdata,
                ggplot2::aes(time.best, bc.cfish.best),
                
                size = plotstyle$linebest$size,
                linetype = plotstyle$linebest$style,
                color = plotstyle$linebest$color) +
    
      ggplot2::geom_line(data = conffitdata,
                ggplot2::aes(time.conf, bc.cfish.conflo),
                
                size = plotstyle$lineconf$size,
                linetype = plotstyle$lineconf$style,
                color = plotstyle$lineconf$color) +
    
      ggplot2::geom_line(data = conffitdata,
                ggplot2::aes(time.conf, bc.cfish.confup),
                
                size = plotstyle$lineconf$size,
                linetype = plotstyle$lineconf$style,
                color = plotstyle$lineconf$color) +
    
      ggplot2::geom_point(shape = plotstyle$symbol$shape,
                 size  = plotstyle$symbol$size,
                 color = plotstyle$symbol$color) +
    
      ggplot2::ggtitle(plottitle) +
    
      ggplot2::labs(x = plotxlab, y = plotylab)
  
  # -- add theme 
  plot2 <- plot1 + expand_theme(plottheme)
  
  print(plot2)
  
}
# ___ (PlotConfFit_Dietary_BoxCox) ____________________


# //////////////////////////////////////////////
# === [PlotConfFit_Dietary_Ln] ====================
# ~~~ assumes BMF.BestFit to exist, saved by RunBestFit_Dietary()
PlotConfFit_Dietary_Ln <- function(testdata    = DATA.Dietary.Work,
                                   testmeas    = TEST.Dietary.Measured,
                                   bestfitdata = BMF.BestFit.Ln,
                                   conffitdata = BMF.ConfFit.Ln,
                                   plotstyle   = PLOT.StyleFit.Ln,
                                   plottheme   = PLOT.Theme,
                                   plottitle   = PLOT.Title.ConfFit.Ln,
                                   plotxlab    = NULL,
                                   plotylab    = NULL) {

  if (is.null(plotxlab)) {
    # -- x label text
    plotxlab <- paste(testmeas["time","name"], "  (",
                      testmeas["time","unit"], ")", sep = "")
  }
  
  if (is.null(plotylab)) {
    # -- y label text
    plotylab <- paste("Ln ", testmeas["cfish","name"], "  (",
                      testmeas["cfish","unit"], ")", sep = "")
  }
  
  #-- scatter of measured test data
  plot0 <- ggplot2::ggplot(testdata, ggplot2::aes(time.data, ln.cfish.data))
  
  # -- add scatter, bestfit curve, and conflo & confup curves
  plot1 <- plot0 +
    
      ggplot2::geom_line(data = bestfitdata,
                ggplot2::aes(time.best, ln.cfish.best),
                
                size = plotstyle$linebest$size,
                linetype = plotstyle$linebest$style,
                color = plotstyle$linebest$color) +
    
      ggplot2::geom_line(data = conffitdata,
                ggplot2::aes(time.conf, ln.cfish.conflo),
                
                size = plotstyle$lineconf$size,
                linetype = plotstyle$lineconf$style,
                color = plotstyle$lineconf$color) +
    
      ggplot2::geom_line(data = conffitdata,
                ggplot2::aes(time.conf, ln.cfish.confup),
                
                size = plotstyle$lineconf$size,
                linetype = plotstyle$lineconf$style,
                color = plotstyle$lineconf$color) +
    
      ggplot2::geom_point(shape = plotstyle$symbol$shape,
                 size  = plotstyle$symbol$size,
                 color = plotstyle$symbol$color) +
    
      ggplot2::ggtitle(plottitle) +
    
      ggplot2::labs(x = plotxlab, y = plotylab)
  
  # -- add theme 
  plot2 <- plot1 + expand_theme(plottheme)
  
  print(plot2)
  
}
# ___ (PlotConfFit_Dietary_Ln) ____________________



# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

# ////////////////////////////////////////////////////////
# === [PlotInvBestFit_Dietary_BoxCox] ====================
# ~~~ assumes BCF.BestFit.Ln to exist, saved by RunBestFit_Dietary()
# ~~~ assumes ln.cfish.data in DATA.Dietary.Work
PlotInvBestFit_Dietary_BoxCox <- function(testdata    = DATA.Dietary.Work,
                                          testmeas    = TEST.Dietary.Measured,
                                          bestfitdata = BMF.BestFit.BoxCox,
                                          plotstyle   = PLOT.StyleFit.BoxCox,
                                          plottheme   = PLOT.Theme,
                                          plottitle   = PLOT.Title.BestFit.BoxCox,
                                          plotxlab    = NULL,
                                          plotylab    = NULL) {

  if (is.null(plotxlab)) {
    # -- x label text
    plotxlab <- paste(testmeas["time","name"], "  (",
                      testmeas["time","unit"], ")", sep = "")
  }
  
  if (is.null(plotylab)) {
    # -- y label text
    plotylab <- paste(testmeas["cfish","name"], "  (",
                      testmeas["cfish","unit"], ")", sep = "")
  }
  
  # -- scatter of measured test data, UN-transformed
  plot0 <- ggplot2::ggplot(testdata, ggplot2::aes(time.data, cfish.data))
  
  # -- add scatter and bestfit curve, back-transformed
  plot1 <- plot0 +
    
      ggplot2::geom_line(data = bestfitdata,
                ggplot2::aes(time.best, inv.bc.cfish.best),
                
                size = plotstyle$linebest$size,
                linetype = plotstyle$linebest$style,
                color = plotstyle$linebest$color) +
    
      ggplot2::geom_point(shape = plotstyle$symbol$shape,
                 size  = plotstyle$symbol$size,
                 color = plotstyle$symbol$color) +
    
      ggplot2::ggtitle(plottitle) +
    
      ggplot2::labs(x = plotxlab, y = plotylab)
  
  # -- add theme 
  plot2 <- plot1 + expand_theme(plottheme)
  
  print(plot2)
  
}
# ___ (PlotInvBestFit_Dietary_BoxCox) ____________________


# ////////////////////////////////////////////////////
# === [PlotInvBestFit_Dietary_Ln] ====================
# ~~~ assumes BCF.BestFit.Ln to exist, saved by RunBestFit_Dietary()
# ~~~ assumes ln.cfish.data in DATA.Dietary.Work
PlotInvBestFit_Dietary_Ln <- function(testdata    = DATA.Dietary.Work,
                                      testmeas    = TEST.Dietary.Measured,
                                      bestfitdata = BMF.BestFit.Ln,
                                      plotstyle   = PLOT.StyleFit.Ln,
                                      plottheme   = PLOT.Theme,
                                      plottitle   = PLOT.Title.BestFit.Ln,
                                      plotxlab    = NULL,
                                      plotylab    = NULL) {
  
  if (is.null(plotxlab)) {
    # -- x label text
    plotxlab <- paste(testmeas["time","name"], "  (",
                      testmeas["time","unit"], ")", sep = "")
  }
  
  if (is.null(plotylab)) {
    # -- y label text
    plotylab <- paste(testmeas["cfish","name"], "  (",
                      testmeas["cfish","unit"], ")", sep = "")
  }
  
  # -- scatter of measured test data, UN-transformed
  plot0 <- ggplot2::ggplot(testdata, ggplot2::aes(time.data, cfish.data))
  
  # -- add scatter and bestfit curve, back-transformed
  plot1 <- plot0 +
    
      ggplot2::geom_line(data = bestfitdata,
                ggplot2::aes(time.best, inv.ln.cfish.best),
                
                size = plotstyle$linebest$size,
                linetype = plotstyle$linebest$style,
                color = plotstyle$linebest$color) +
    
      ggplot2::geom_point(shape = plotstyle$symbol$shape,
                 size  = plotstyle$symbol$size,
                 color = plotstyle$symbol$color) +
    
      ggplot2::ggtitle(plottitle) +
    
      ggplot2::labs(x = plotxlab, y = plotylab)
  
  # -- add theme 
  plot2 <- plot1 + expand_theme(plottheme)
  
  print(plot2)
  
}
# ___ (PlotInvBestFit_Dietary_Ln) ____________________


# ////////////////////////////////////////////////////
# === [PlotInvConfFit_Dietary_BoxCox] ====================
# ~~~ assumes BCF.BestFit.Ln to exist, saved by RunBestFit_Dietary()
# ~~~ assumes ln.cfish.data in DATA.Dietary.Work
PlotInvConfFit_Dietary_BoxCox <- function(testdata    = DATA.Dietary.Work,
                                          testmeas    = TEST.Dietary.Measured,
                                          bestfitdata = BMF.BestFit.BoxCox,
                                          conffitdata = BMF.ConfFit.BoxCox,
                                          plotstyle   = PLOT.StyleFit.BoxCox,
                                          plottheme   = PLOT.Theme,
                                          plottitle   = PLOT.Title.ConfFit.BoxCox,
                                          plotxlab    = NULL,
                                          plotylab    = NULL) {
  
  if (is.null(plotxlab)) {
    # -- x label text
    plotxlab <- paste(testmeas["time","name"], "  (",
                      testmeas["time","unit"], ")", sep = "")
  }
  
  if (is.null(plotylab)) {
    # -- y label text
    plotylab <- paste(testmeas["cfish","name"], "  (",
                      testmeas["cfish","unit"], ")", sep = "")
  }
  
  # -- scatter of measured test data, UN-transformed
  plot0 <- ggplot2::ggplot(testdata, ggplot2::aes(time.data, cfish.data))
  
  # -- add scatter and bestfit curve, back-transformed
  plot1 <- plot0 +
    
      ggplot2::geom_line(data = bestfitdata,
                ggplot2::aes(time.best, inv.bc.cfish.best),
                
                size = plotstyle$linebest$size,
                linetype = plotstyle$linebest$style,
                color = plotstyle$linebest$color) +
    
      ggplot2::geom_line(data = conffitdata,
                ggplot2::aes(time.conf, inv.bc.cfish.conflo),
                
                size = plotstyle$lineconf$size,
                linetype = plotstyle$lineconf$style,
                color = plotstyle$lineconf$color) +
    
      ggplot2::geom_line(data = conffitdata,
                ggplot2::aes(time.conf, inv.bc.cfish.confup),
                
                size = plotstyle$lineconf$size,
                linetype = plotstyle$lineconf$style,
                color = plotstyle$lineconf$color) +
    
      ggplot2::geom_point(shape = plotstyle$symbol$shape,
                 size  = plotstyle$symbol$size,
                 color = plotstyle$symbol$color) +
    
      ggplot2::ggtitle(plottitle) +
    
      ggplot2::labs(x = plotxlab, y = plotylab)
    
  # -- add theme 
  plot2 <- plot1 + expand_theme(plottheme)
  
  print(plot2)
  
}
# ___ (PlotInvConfFit_Dietary_BoxCox) ____________________


# ////////////////////////////////////////////////////
# === [PlotInvConfFit_Dietary_Ln] ====================
# ~~~ assumes BCF.BestFit.Ln to exist, saved by RunBestFit_Dietary()
# ~~~ assumes ln.cfish.data in DATA.Dietary.Work
PlotInvConfFit_Dietary_Ln <- function(testdata    = DATA.Dietary.Work,
                                      testmeas    = TEST.Dietary.Measured,
                                      bestfitdata = BMF.BestFit.Ln,
                                      conffitdata = BMF.ConfFit.Ln,
                                      plotstyle   = PLOT.StyleFit.Ln,
                                      plottheme   = PLOT.Theme,
                                      plottitle   = PLOT.Title.ConfFit.Ln,
                                      plotxlab    = NULL,
                                      plotylab    = NULL) {
  
  if (is.null(plotxlab)) {
    # -- x label text
    plotxlab <- paste(testmeas["time","name"], "  (",
                      testmeas["time","unit"], ")", sep = "")
  }
  
  if (is.null(plotylab)) {
    # -- y label text
    plotylab <- paste(testmeas["cfish","name"], "  (",
                      testmeas["cfish","unit"], ")", sep = "")
  }
  
  # -- scatter of measured test data, UN-transformed
  plot0 <- ggplot2::ggplot(testdata, ggplot2::aes(time.data, cfish.data))
  
  # -- add scatter and bestfit curve, back-transformed
  plot1 <- plot0 + 
    
      ggplot2::geom_line(data = bestfitdata,
                ggplot2::aes(time.best, inv.ln.cfish.best),
                
                size = plotstyle$linebest$size,
                linetype = plotstyle$linebest$style,
                color = plotstyle$linebest$color) +
  
      ggplot2::geom_line(data = conffitdata,
                ggplot2::aes(time.conf, inv.ln.cfish.conflo),
                
                size = plotstyle$lineconf$size,
                linetype = plotstyle$lineconf$style,
                color = plotstyle$lineconf$color) +
  
      ggplot2::geom_line(data = conffitdata,
                ggplot2::aes(time.conf, inv.ln.cfish.confup),
                
                size = plotstyle$lineconf$size,
                linetype = plotstyle$lineconf$style,
                color = plotstyle$lineconf$color) +
      
      ggplot2::geom_point(shape = plotstyle$symbol$shape,
                 size  = plotstyle$symbol$size,
                 color = plotstyle$symbol$color) +
    
      ggplot2::ggtitle(plottitle) +
    
      ggplot2::labs(x = plotxlab, y = plotylab)
  
  # -- add theme 
  plot2 <- plot1 + expand_theme(plottheme)
  
  print(plot2)
  
}
# ___ (PlotInvConfFit_Dietary_Ln) ____________________



# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

# /////////////////////////////////////////////////////
# # === [plotBoxCoxLambda_Dietary] ====================
# plotBoxCoxLambda_Dietary <- function(modelfit = BMF.ModelFit) {
#   
#   bcmf <- boxcox2.nls(modelfit)
#   
#   lambdafit <- bcmf$lambda$lambda
#   
#   lambdaconf <- bcmf$lambda$ci
#   
#   lambdaest <- c(lambdafit, lambdaconf)
#   names(lambdaest) <- c("lambda", "conflo", "confup")  # "fit"??
#   
#   return(lambdaest)
# }
# # ___ (plotBoxCoxLambda_Dietary) ____________________



# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

# /////////////////////////////////////////////
# === [RunBestFit_Dietary] ====================
# --- direct implementation
RunBestFit_Dietary <- function(datawork = DATA.Dietary.Work,
                               paramfit = BMF.ParamFit,
                               savefit = TRUE,
                               steps   = 60,
                               tmin    = NULL,
                               tmax    = NULL,
                               textra  = NULL) {
  
  timesumm <- dataTime_Summary(datawork)
  
  if (is.null(tmin)) {
    tmin <- timesumm["min"]
  }
  
  if (is.null(tmax)) {
    tmax <- timesumm["max"]
  }
  
  if (!is.null(textra)) {
    trange <- tmax - tmin
    textend <- textra*trange
    
    tmin <- max(1e-9, tmin - textend)
    tmax <- tmax + textend
  }
  
  timepred <- seq(tmin, tmax, length.out = steps + 1)
  
  cfishpred <- RunModel_Dietary(timepred, paramfit["c0d"], paramfit["k2"])
  
  predframe <- data.frame(time.best = timepred, cfish.best = cfishpred)
  
  if (savefit)
    assign("BMF.BestFit", predframe, envir = .GlobalEnv)
  
  top    <- head(predframe, n=10)
  bottom <- tail(predframe, n=10)
  
  print(top)
  cat("   ...")
  cat("\n")
  print(bottom)
  
  #return(predframe)
  
}
# ___ (RunBestFit_Dietary) ____________________


# ////////////////////////////////////////////////////
# === [RunBestFit_Dietary_BoxCox] ====================
# --- direct implementation
RunBestFit_Dietary_BoxCox <- function(datawork = DATA.Dietary.Work,
                                      parfitbc = BMF.ParamFit.BoxCox,
                                      lambda   = BMF.ParamFit.Lambda,
                                      lnstarter = 1e-9,
                                      savefit = TRUE,
                                      steps   = 60,
                                      tmin    = NULL,
                                      tmax    = NULL,
                                      textra  = NULL) {
  
  timesumm <- dataTime_Summary(datawork)
  
  if (is.null(tmin)) {
    tmin <- timesumm["min"]
  }
  
  if (is.null(tmax)) {
    tmax <- timesumm["max"]
  }
  
  if (!is.null(textra)) {
    trange <- tmax - tmin
    textend <- textra*trange
    
    tmin <- max(1e-9, tmin - textend)
    tmax <- tmax + textend
  }
  
  # -- no need to add tdepur as in Aqueous
  timepred <- seq(tmin, tmax, length.out = steps + 1)
  
  # -- untransformed modelrun on bc-transformed fitted params
  invbccfishpred <- RunModel_Dietary(timepred, parfitbc["c0d"], parfitbc["k2"])
  
  # -- the bcpow-transformed prediction is just that
  bccfishpred <- car::bcPower(invbccfishpred + lnstarter, lambda)
  
  # -- put best fit into dataframe
  predframe <- data.frame(time.best = timepred,
                          bc.cfish.best = bccfishpred,
                          inv.bc.cfish.best = invbccfishpred)
  
  # -- save it
  if (savefit)
    assign("BMF.BestFit.BoxCox", predframe, envir = .GlobalEnv)
  
  top    <- head(predframe, n=10)
  bottom <- tail(predframe, n=10)
  
  print(top)
  cat("   ...")
  cat("\n")
  print(bottom)
  
  #return(predframe)
  
}
# ___ (RunBestFit_Dietary_BoxCox) ____________________


# /////////////////////////////////////////////
# === [RunBestFit_Dietary_Ln] ====================
# --- direct implementation
RunBestFit_Dietary_Ln <- function(datawork = DATA.Dietary.Work,
                                  parfitln = BMF.ParamFit.Ln,
                                  lnstarter = 1e-9,
                                  savefit = TRUE,
                                  steps   = 60,
                                  tmin    = NULL,
                                  tmax    = NULL,
                                  textra  = NULL) {
  
  timesumm <- dataTime_Summary(datawork)
  
  if (is.null(tmin)) {
    tmin <- timesumm["min"]
  }
  
  if (is.null(tmax)) {
    tmax <- timesumm["max"]
  }
  
  if (!is.null(textra)) {
    trange <- tmax - tmin
    textend <- textra*trange
    
    tmin <- max(1e-9, tmin - textend)
    tmax <- tmax + textend
  }
  
  # -- no tdepur to add, as in Aqueous
  timepred <- seq(tmin, tmax, length.out = steps + 1)
  
  # -- untransformed modelrun on ln-transformed fitted params
  invlncfishpred <- RunModel_Dietary(timepred, parfitln["c0d"], parfitln["k2"])
  
  # -- the ln-transformed prediction is just the log
  lncfishpred <- log(invlncfishpred + lnstarter)
  
  # -- put in dataframe with appropriate names, including inverse
  predframe <- data.frame(time.best = timepred,
                          ln.cfish.best = lncfishpred,
                          inv.ln.cfish.best = invlncfishpred)
  
  if (savefit)
    assign("BMF.BestFit.Ln", predframe, envir = .GlobalEnv)
  
  top    <- head(predframe, n=10)
  bottom <- tail(predframe, n=10)
  
  print(top)
  cat("   ...")
  cat("\n")
  print(bottom)
  
  #return(predframe)
}
# ___ (RunBestFit_Dietary_Ln) ____________________



# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

# /////////////////////////////////////////////
# === [RunConfFit_Dietary] ====================
RunConfFit_Dietary <- function(datawork  = DATA.Dietary.Work,
                               paramboot = BMF.ParamBoot,
                               savefit   = TRUE,
                               steps     = 60,
                               tmin      = NULL,
                               tmax      = NULL,
                               textra    = NULL) {
  
  timesumm <- dataTime_Summary(datawork)
  
  if (is.null(tmin)) {
    tmin <- timesumm["min"]
  }
  
  if (is.null(tmax)) {
    tmax <- timesumm["max"]
  }
  
  if (!is.null(textra)) {
    trange <- tmax - tmin
    textend <- textra*trange
    
    tmin <- max(1e-9, tmin - textend)
    tmax <- tmax + textend
  }
  
  # -- prep timeseries
  timepred <- seq(tmin, tmax, length.out = steps + 1)
  
  # -- how many boot instances
  lenboot <- length(row.names(paramboot))
  
  numtimes <- length(timepred)
  
  # -- declare space for conf estimates
  modconflo <- numeric(numtimes)
  modconfup <- numeric(numtimes)
  
 
  for (i in 1:numtimes) {
    
    timerepeat <- rep(timepred[i], lenboot)
  
    cfishboot <- RunModel_Dietary(timerepeat, paramboot$c0d.boot, paramboot$k2.boot)
    
    modconflo[i] <- quantile(cfishboot, 0.025, type = 8)  # Tukey?
    modconfup[i] <- quantile(cfishboot, 0.975, type = 8)
    
  }
  
  # -- put into dataframe with appropriate names
  confframe <- data.frame(time.conf = timepred,
                          cfish.conflo = modconflo,
                          cfish.confup = modconfup)
  
  if (savefit)
    assign("BMF.ConfFit", confframe, envir = .GlobalEnv)
  
  top    <- head(confframe, n = 10)
  bottom <- tail(confframe, n = 10)
  
  print(top)
  cat("   ...")
  cat("\n")
  print(bottom)
  
  # return(confframe)
}
# ___ (RunConfFit_Dietary) ____________________


# ////////////////////////////////////////////////
# === [RunConfFit_Dietary_BoxCox] ====================
RunConfFit_Dietary_BoxCox <- function(datawork = DATA.Dietary.Work,
                                      paramboot = BMF.ParamBoot.BoxCox,
                                      lambda    = BMF.ParamFit.Lambda,
                                      lnstarter = 1e-9,
                                      savefit   = TRUE,
                                      steps     = 60,
                                      tmin      = NULL,
                                      tmax      = NULL,
                                      textra    = NULL) {
  
  timesumm <- dataTime_Summary(datawork)
  
  if (is.null(tmin)) {
    tmin <- timesumm["min"]
  }
  
  if (is.null(tmax)) {
    tmax <- timesumm["max"]
  }
  
  if (!is.null(textra)) {
    trange <- tmax - tmin
    textend <- textra*trange
    
    tmin <- max(1e-9, tmin - textend)
    tmax <- tmax + textend
  }
  
  # -- prep timeseries
  timepred <- seq(tmin, tmax, length.out = steps + 1)
  
  # -- how many boot instances
  lenboot <- length(row.names(paramboot))
  
  numtimes <- length(timepred)
  
  # -- declare space for conf estimates: backtransformed
  invbcmodconflo <- numeric(numtimes)
  invbcmodconfup <- numeric(numtimes)
  
  # -- bc-transformed
  bcmodconflo <- numeric(numtimes)
  bcmodconfup <- numeric(numtimes)
  
  
  for (i in 1:numtimes) {
    
    timerepeat <- rep(timepred[i], lenboot)
    
    # -- untransformed modelrun on ln-transformed fitted params
    invbccfishboot <- RunModel_Dietary(timerepeat,
                                       paramboot$c0d.boot.bc, paramboot$k2.boot.bc)
    
    bccfishboot <- car::bcPower(invbccfishboot + lnstarter, lambda)
    
    invbcmodconflo[i] <- quantile(invbccfishboot, 0.025, type = 8)
    invbcmodconfup[i] <- quantile(invbccfishboot, 0.975, type = 8)
    
    bcmodconflo[i] <- quantile(bccfishboot, 0.025, type = 8)
    bcmodconfup[i] <- quantile(bccfishboot, 0.975, type = 8)
    
  }
  
  # -- put into dataframe with appropriate names
  confframe <- data.frame(time.conf = timepred,
                          bc.cfish.conflo = bcmodconflo,
                          bc.cfish.confup = bcmodconfup,
                          inv.bc.cfish.conflo = invbcmodconflo,
                          inv.bc.cfish.confup = invbcmodconfup)
  
  if (savefit)
    assign("BMF.ConfFit.BoxCox", confframe, envir = .GlobalEnv)
  
  top    <- head(confframe, n = 10)
  bottom <- tail(confframe, n = 10)
  
  print(top)
  cat("   ...")
  cat("\n")
  print(bottom)
  
  # return(confframe)
}
# ___ (RunConfFit_Dietary_BoxCox) ____________________


# ////////////////////////////////////////////////
# === [RunConfFit_Dietary_Ln] ====================
RunConfFit_Dietary_Ln <- function(datawork  = DATA.Dietary.Work,
                                  paramboot = BMF.ParamBoot.Ln,
                                  lnstarter = 1e-9,
                                  savefit   = TRUE,
                                  steps     = 60,
                                  tmin      = NULL,
                                  tmax      = NULL,
                                  textra    = NULL) {
  
  timesumm <- dataTime_Summary(datawork)
  
  if (is.null(tmin)) {
    tmin <- timesumm["min"]
  }
  
  if (is.null(tmax)) {
    tmax <- timesumm["max"]
  }
  
  if (!is.null(textra)) {
    trange <- tmax - tmin
    textend <- textra*trange
    
    tmin <- max(1e-9, tmin - textend)
    tmax <- tmax + textend
  }
  
  # -- prep timeseries
  timepred <- seq(tmin, tmax, length.out = steps + 1)
  
  # -- how many boot instances
  lenboot <- length(row.names(paramboot))
  
  numtimes <- length(timepred)
  
  # -- declare space for conf estimates: backtransformed
  invlnmodconflo <- numeric(numtimes)
  invlnmodconfup <- numeric(numtimes)
  
  # -- ln-transformed
  lnmodconflo <- numeric(numtimes)
  lnmodconfup <- numeric(numtimes)

  
  for (i in 1:numtimes) {
    
    timerepeat <- rep(timepred[i], lenboot)
    
    # -- untransformed modelrun on ln-transformed fitted params
    invlncfishboot <- RunModel_Dietary(timerepeat,
                                       paramboot$c0d.boot.ln, paramboot$k2.boot.ln)
    
    lncfishboot <- log(invlncfishboot + lnstarter)
    
    invlnmodconflo[i] <- quantile(invlncfishboot, 0.025, type = 8)
    invlnmodconfup[i] <- quantile(invlncfishboot, 0.975, type = 8)
    
    lnmodconflo[i] <- quantile(lncfishboot, 0.025, type = 8)
    lnmodconfup[i] <- quantile(lncfishboot, 0.975, type = 8)
    
  }
  
  # -- put into dataframe with appropriate names
  confframe <- data.frame(time.conf = timepred,
                          ln.cfish.conflo = lnmodconflo,
                          ln.cfish.confup = lnmodconfup,
                          inv.ln.cfish.conflo = invlnmodconflo,
                          inv.ln.cfish.confup = invlnmodconfup)
  
  if (savefit)
    assign("BMF.ConfFit.Ln", confframe, envir = .GlobalEnv)
  
  top    <- head(confframe, n = 10)
  bottom <- tail(confframe, n = 10)
  
  print(top)
  cat("   ...")
  cat("\n")
  print(bottom)
  
  # return(confframe)
}
# ___ (RunConfFit_Dietary_Ln) ____________________



# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

# === [RunModel_Dietary] ====================
# --- FINAL bcmfR
# --- uses C0,d as parameter: no testparam as input
RunModel_Dietary <- function(time, parc0d, park2) {
  
  cfish <- parc0d*exp(-park2*time)
  
  return(cfish)
}
# ___ (RunModel_Dietary) ____________________


# /////////////////
# === [RunModel_Dietary_BoxCox] ====================
# --- FINAL bcmfR
# --- uses C0,d as parameter: no testparam as input
RunModel_Dietary_BoxCox <- function(time, parc0d, park2,
                                    lambda, lnstarter = 1e-9) {
  
  bccfish <- car::bcPower(parc0d*exp(-park2*time) + lnstarter, lambda)
  
  return(bccfish)
}
# ___ (RunModel_Dietary_BoxCox) ____________________


# /////////////////

# === [RunModel_Dietary_Ln] ====================
# --- FINAL bcmfR
# --- uses C0,d as parameter: no testparam as input
RunModel_Dietary_Ln <- function(time, parc0d, park2,
                                lnstarter = 1e-9) {
  
  lncfish <- log(parc0d*exp(-park2*time) + lnstarter)
  
  return(lncfish)
}
# ___ (RunModel_Dietary_Ln) ____________________



# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

# ///////////////////////////////////////////
# === [StartFit_Dietary] ====================
# --- FINAL bcmfR
# --- startFit calculates a startfit vector (not a list)
StartFit_Dietary <-  function(datacfish = DATA.Dietary.Work,
                              #testparam = TEST.Dietary.Design,
                              lnstarter = 1e-9) {
  
  #-- fit linear model to the lndepur data
  linmodlndepur <- lm(log(cfish.data + lnstarter) ~ time.data, data = datacfish)
  
  #-- coefficients
  coefvec <- coef(linmodlndepur)
  
  beta0 <- coefvec[1]
  beta1 <- coefvec[2]
  
  #-- k2 is minus beta1 (slope)
  k2 <- -beta1
  
  #-- c0d at 0 is exponential of ln intercept
  c0d <- exp(beta0)
  
  #-- 
  startfit <- c(c0d, k2)
  names(startfit) <- c("fitc0d", "fitk2")

  return(startfit)
}
# ___ (StartFit_Dietary) ____________________



# @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

# === [SummaryTable_Dietary] ====================
SummTable_Dietary <-  function(modelfit  = BMF.ModelFit,
                                  testparam = TEST.Dietary.Design,
                                  savefit   = TRUE,
                                  digits    = NUM.Digits) {
  
  #-- call generic SummaryTable_Dietary
  summtabdiet <- SummaryTable_Dietary_GENERIC(modelfit, testparam)
  
  #-- store in global var
  if (savefit)
    assign("BMF.SummTable", summtabdiet, envir = .GlobalEnv)
  
  #-- return estimation dataframe
  if (is.null(digits)) {
    summtabdiet
  } else {
    lucid::lucid(summtabdiet, dig = digits)
  }
  
}
# ___ (SummaryTable_Dietary) ____________________


# /////////////////

# === [SummaryTable_Dietary_BoxCox] ====================
SummTable_Dietary_BoxCox <-  function(modelfit  = BMF.ModelFit.BoxCox,
                                         testparam = TEST.Dietary.Design,
                                         savefit   = TRUE,
                                         digits    = NUM.Digits) {
  
  #-- call generic SummaryTable_Dietary
  summtabdiet <- SummaryTable_Dietary_GENERIC(modelfit, testparam)
  
  #-- store in global var
  if (savefit)
    assign("BMF.SummTable.BoxCox", summtabdiet, envir = .GlobalEnv)
  
  #-- return estimation dataframe
  if (is.null(digits)) {
    summtabdiet
  } else {
    lucid::lucid(summtabdiet, dig = digits)
  }
  
}
# ___ (SummaryTable_Dietary_BoxCox) ____________________


# /////////////////

# === [SummaryTable_Dietary_Ln] ====================
SummTable_Dietary_Ln <-  function(modelfit  = BMF.ModelFit.Ln,
                                     testparam = TEST.Dietary.Design,
                                     savefit   = TRUE,
                                     digits    = NUM.Digits) {
  
  #-- call generic SummaryTable_Dietary
  summtabdiet <- SummaryTable_Dietary_GENERIC(modelfit, testparam)
  
  #-- store in global var
  if (savefit)
    assign("BMF.SummTable.Ln", summtabdiet, envir = .GlobalEnv)
  
  #-- return estimation dataframe
  if (is.null(digits)) {
    summtabdiet
  } else {
    lucid::lucid(summtabdiet, dig = digits)
  }
  
}
# ___ (SummaryTable_Dietary_Ln) ____________________


# /////////////////

# === [SummaryTable_Dietary_GENERIC] ====================
# --- relates to the C0d, K2 parameterization only
# --- CHANGE modelfit to more general modelfit object name
SummaryTable_Dietary_GENERIC <-  function(modelfit,
                                          testparam = TEST.Dietary.Design) {
  
  #-- get working parameter values from input param vector
  testcfood <- testparam["cfood", "value"]
  timefeed  <- testparam["tfeed", "value"]
  ingestion <- testparam["ingestion", "value"]
  kgrowth   <- testparam["kgrowth", "value"]
  
  lipcorr   <- testparam["lipidfish", "value"]/testparam["lipidfood", "value"]
  
  
  #-- parnames must be the nls fitparameter names!!
  parnames <- c("fitc0d","fitk2")
  
  
  gfuncc0d <- "fitc0d"
  
  #-- kd (depuration, backnamed k2)
  gfunckd <- "fitk2"
  
  #-- ke elimination rate constant: k2 - kg, follow OECD305 k2g
  gfuncke <- paste("fitk2-", kgrowth, sep = "")
  
  #-- growth-corrected half-life
  gfuncthg <- paste(0.693, "/(fitk2-", kgrowth,")", sep = "")
  
  #-- kf uptake rate constant from food
  gfunckf <- paste("fitc0d/", testcfood,
                   "*fitk2/(1-exp(-fitk2*", timefeed, "))", sep = "") 
  
  #-- bmf = kf/kd
  gfuncbmf <- paste("fitc0d/", testcfood,
                    "/(1-exp(-fitk2*", timefeed, "))", sep = "")
  
  #-- bmfg = kf/(kd - kg)
  gfuncbmfg <- paste("fitc0d/", testcfood,
                     "*fitk2/(fitk2-", kgrowth,")/(1-exp(-fitk2*", timefeed, "))", sep = "")
  
  #-- BMFKgL lipid corrected
  gfuncbmfgl <- paste("fitc0d/", testcfood,"/", lipcorr,
                      "*fitk2/(fitk2-", kgrowth,")/(1-exp(-fitk2*", timefeed, "))", sep = "")
  
  
  #-- alpha is chemical assimilation efficiency
  gfuncalpha <- paste("fitc0d/", testcfood,
                      "*fitk2/(1-exp(-fitk2*", timefeed, "))/", ingestion, sep = "")
  
  
  
  #-- deltaMethod from package car for c0d
  delmethc0d   <- car::deltaMethod(modelfit, g = gfuncc0d,
                              parameterNames = parnames)
  
  #-- deltaMethod from package car for kd
  delmethkd    <- car::deltaMethod(modelfit, g = gfunckd,
                              parameterNames = parnames)
  
  #-- deltaMethod from package car for ke
  delmethke    <- car::deltaMethod(modelfit, g = gfuncke,
                              parameterNames = parnames)
  
  #-- deltaMethod from package car for thg (growth-corrected half-life)
  delmeththg  <- car::deltaMethod(modelfit, g = gfuncthg,
                             parameterNames = parnames)
  
  #-- deltaMethod from package car for kf
  delmethkf    <- car::deltaMethod(modelfit, g = gfunckf,
                              parameterNames = parnames)
  
  #-- deltaMethod from package car for bmf
  delmethbmf   <- car::deltaMethod(modelfit, g = gfuncbmf,
                              parameterNames = parnames)
  
  #-- deltaMethod from package car for bmfg
  delmethbmfg  <- car::deltaMethod(modelfit, g = gfuncbmfg,
                              parameterNames = parnames)
  
  #-- deltaMethod from package car for bmfgl
  delmethbmfgl  <- car::deltaMethod(modelfit, g = gfuncbmfgl,
                               parameterNames = parnames)
  
  #-- deltaMethod from package car for alpha
  delmethalpha <- car::deltaMethod(modelfit, g = gfuncalpha,
                              parameterNames = parnames)
  
  
  #-- collect individual estimates and standard errors
  estc0d  <- delmethc0d[[1]]
  estkd   <- delmethkd[[1]]
  estke   <- delmethke[[1]]
  estthg  <- delmeththg[[1]]
  estkf   <- delmethkf[[1]]
  estbmf  <- delmethbmf[[1]]
  estbmfg <- delmethbmfg[[1]]
  estbmfgl <- delmethbmfgl[[1]]
  estalpha <- delmethalpha[[1]]
  
  #-- and standard errors
  errc0d  <- delmethc0d[[2]]
  errkd   <- delmethkd[[2]]
  errke   <- delmethke[[2]]
  errthg  <- delmeththg[[2]]
  errkf   <- delmethkf[[2]]
  errbmf  <- delmethbmf[[2]]
  errbmfg <- delmethbmfg[[2]]
  errbmfgl <- delmethbmfgl[[2]]
  erralpha <- delmethalpha[[2]]
  
  #-- build Wald confidence intervals
  cloc0d <- estc0d - 1.96*errc0d
  cupc0d <- estc0d + 1.96*errc0d
  
  clokd <- estkd - 1.96*errkd
  cupkd <- estkd + 1.96*errkd
  
  cloke <- estke - 1.96*errke
  cupke <- estke + 1.96*errke
  
  clothg <- estthg - 1.96*errthg
  cupthg <- estthg + 1.96*errthg
  
  clokf <- estkf - 1.96*errkf
  cupkf <- estkf + 1.96*errkf
  
  clobmf <- estbmf - 1.96*errbmf
  cupbmf <- estbmf + 1.96*errbmf
  
  clobmfg <- estbmfg - 1.96*errbmfg
  cupbmfg <- estbmfg + 1.96*errbmfg
  
  clobmfgl <- estbmfgl - 1.96*errbmfgl
  cupbmfgl <- estbmfgl + 1.96*errbmfgl
  
  cloalpha <- estalpha - 1.96*erralpha
  cupalpha <- estalpha + 1.96*erralpha
  
  
  unitnames <- c("mgX/kgFish", "1/day", "1/day", "kgFood/kgFish/day", "-",
                 "kgFood/kgFish", "kgFood/kgFish", "day", "kgFood/kgFish")
  
  
  #-- put the values in a data frame
  dfbmfpar <- data.frame(c(estc0d, estkd, estke, estkf, estalpha, estbmf, estbmfg, estthg, estbmfgl),
                         c(errc0d, errkd, errke, errkf, erralpha, errbmf, errbmfg, errthg, errbmfgl),
                         c(cloc0d, clokd, cloke, clokf, cloalpha, clobmf, clobmfg, clothg, clobmfgl),
                         c(cupc0d, cupkd, cupke, cupkf, cupalpha, cupbmf, cupbmfg, cupthg, cupbmfgl),
                         unitnames)
  
  #-- set header names
  names(dfbmfpar) <- c("Estimate", "Std.Error", "2.5%", "97.5%", "unit")
  
  #-- set parameter row names
  row.names(dfbmfpar) <- c("C0d", "k2", "k2g", "kf", "alpha", "BMFK", "BMFKg", "tHalfg", "BMFKgLipid")
  
  return(dfbmfpar)
  
}
# ___ (SummaryTable_Dietary_GENERIC) ____________________




# %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%